/*
Author: Jelle Geerts

Usage of the works is permitted provided that this instrument is
retained with the works, so that any entity that uses the works is
notified of this instrument.

DISCLAIMER: THE WORKS ARE WITHOUT WARRANTY.
*/

function KeePassHelper_PrefMan() {
    var ROOT_BRANCH = "extensions.keepasshelper.",
        pref = Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefService).getBranch(ROOT_BRANCH),
        defaultPref = Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefService).getDefaultBranch(ROOT_BRANCH),
        observers = [],
        getDefaultValue,
        getValue,
        getUrlMode;

    // Retrieve a list of all preferences.
    KeePassHelper_PrefMan.list = function () {
        return pref.getChildList("", {});
    };

    KeePassHelper_PrefMan.exists = function (name) {
        return pref.getPrefType(name) !== pref.PREF_INVALID;
    };

    this.getDefaultValue = function (name) {
        var type, value;

        try {
            type = defaultPref.getPrefType(name);
            if (type === defaultPref.PREF_INVALID) {
                dump("[KeePass Helper] PrefMan: failed retrieving type for *default* preference `" + name + "'.\n");
            } else {
                switch (type) {
                case defaultPref.PREF_STRING:
                    value = defaultPref.getCharPref(name);
                    break;
                case defaultPref.PREF_BOOL:
                    value = defaultPref.getBoolPref(name);
                    break;
                case defaultPref.PREF_INT:
                    value = defaultPref.getIntPref(name);
                    break;
                }
            }
        } catch (ex) {
            dump("[KeePass Helper] PrefMan: failed retrieving *default* value for preference `" + name + "'.\n");
        }

        if (typeof value !== "undefined") {
            return value;
        }

        return null;
    };

    this.getValue = function (name, valueOnFail) {
        var type, value;

        try {
            type = pref.getPrefType(name);
            if (type === pref.PREF_INVALID) {
                dump("[KeePass Helper] PrefMan: failed retrieving type for preference `" + name + "'.\n");
            } else {
                switch (type) {
                case pref.PREF_STRING:
                    value = pref.getCharPref(name);
                    break;
                case pref.PREF_BOOL:
                    value = pref.getBoolPref(name);
                    break;
                case pref.PREF_INT:
                    value = pref.getIntPref(name);
                    break;
                }
            }
        } catch (ex) {
            dump("[KeePass Helper] PrefMan: failed retrieving value for preference `" + name + "'.\n");
        }

        if (typeof value !== "undefined") {
            return value;
        }

        return (typeof valueOnFail !== "undefined" ? valueOnFail : null);
    };

    this.destroy = function () {
        var i;

        for (i = 0; i < observers.length; ++i) {
            pref.removeObserver(observers[i].domain, observers[i]);
        }
    };

    this.addObserver = function (name, observer) {
        var observerWrapper,
            _this = this;

        if (typeof observers[observer] !== "undefined" && observers[observer] !== null) {
            dump("[KeePass Helper] PrefMan: tried to add an observer that was already registered.\n");
            return;
        }

        observerWrapper = {
            domain: name,
            observe: function (subject, topic, data) {
                observer(_this.getValue(name));
            }
        };

        observers[observer] = observerWrapper;

        pref.QueryInterface(Components.interfaces.nsIPrefBranch2)
            .addObserver(name, observerWrapper, false);
    };

    this.removeObserver = function (observer) {
        if (!observers[observer]) {
            dump("[KeePass Helper] PrefMan: tried to remove an observer that wasn't registered.\n");
            return;
        }

        pref.QueryInterface(Components.interfaces.nsIPrefBranch2)
            .removeObserver(observers[observer].domain, observers[observer]);

        observers[observer] = null;
    };

    this.getUrlMode = function () {
        var value = this.getValue("url_mode"),
            valid = ["hostname", "short", "full"];

        if (valid.indexOf(value) === -1) {
            // Value is invalid. Get default value instead.
            value = this.getDefaultValue("url_mode");
        }

        return value;
    };
}
